
<!DOCTYPE html>
<html ng-app="Static">
<head>
    <meta charset="UTF-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <title>Learning Assemble, Step 12 - Navigation Helper | James Wing</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta name="description" content="Part of a series exploring the Assemble static web site framework for Node.js" />

    

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/font-awesome/4.1.0/css/font-awesome.min.css"/>

<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/css/bootstrap.min.css" />



<link rel="stylesheet" type="text/css" href="../assets/css/compiled/site.css">



<!--<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/highlight.js/8.6/styles/default.min.css">-->


<link href="../assets/prism/prism.css" rel="stylesheet" />

<!--[if lt IE 9]>
<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->

<script src="//ajax.googleapis.com/ajax/libs/jquery/1.10.2/jquery.min.js"></script>


</head>
<body id="blogpost" ng-controller="BaseController">



<header class="navbar navbar-inverse normal" role="banner">
    <div class="navbar-container">
        <div class="navbar-header">
            <button class="navbar-toggle" type="button" data-toggle="collapse" data-target=".bs-navbar-collapse">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a href="/index.html" class="navbar-brand">James Wing</a>
        </div>
        <nav class="collapse navbar-collapse bs-navbar-collapse" role="navigation">
            <ul class="nav navbar-nav navbar-right">
                

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


            </ul>
        </nav>
    </div>
</header>



<div class="layout-sidebar">
    <div class="row">
        <div class="col-sm-12 col-md-9 col-lg-8">
            

<div class="blogpost-wrapper">
    

    <div class="row">
        <div class="col-md-10 col-lg-10 post">
            <h1 class="title">
                <a href="">Learning Assemble, Step 12 - Navigation Helper</a>
            </h1>
            <!--<div class="author">-->
                <!---->
                <!---->
                <!---->
                <!--2015-02-06 by James Wing-->
            <!--</div>-->
            <div class="date">
                2015-02-06
            </div>

            

            <div class="content">

                
                    <p>As part of my recent struggle <a href="/learning-assemble/step-11-more-navigation.html">building a list of 5 recent blog posts</a>,
I decided that I should just write a custom Handlebars Helper that did what I wanted.
After I wrote the helper, I figured out that there was actually a built-in way to accomplish the task,
such that my custom code became an unnecessary and possibly buggy addition.  I was very disappointed.</p>
<p>But I felt obligated to flesh out my <strong>#withPages</strong> helper and share it.
It is based on the built-in <strong>#withSort</strong> helper code, with some basic lodash action on the pages array,
and a light topping of error messages.  But it could be handy.</p>

                    <p>A typical usage would be something like the following:</p>
<pre><code class="language-markup">&lt;h2&gt;Recent Posts by Posted Date&lt;/h2&gt;
&lt;ul&gt;
    {{#withPages collection=&quot;posts&quot; inflection=&quot;post&quot; value=&quot;all&quot; sortBy=&quot;data.posted&quot; dir=&quot;desc&quot; limit=5}}
        &lt;li&gt;{{formatDate data.posted &quot;%F&quot;}}: Post - {{data.title}} ({{basename}})&lt;/li&gt;
    {{/withPages}}
&lt;/ul&gt;</code></pre>
<p>the arguments allow for filtering collections, sorting, and limiting the pages:</p>
<p><strong>Collection Filtering</strong></p>
<p>Filtering pages in a collection requires three arguments.  By default, {{#withPages}} uses all pages.</p>
<ul>
<li><code>collection</code> - the name of the collection defined in the Gruntfile (typically &quot;tags&quot;, &quot;posts&quot;, &quot;keywords&quot;, etc.)</li>
<li><code>inflection</code> - the inflection for the collection, defined in the Gruntfile (&quot;tag&quot;, &quot;post&quot;, &quot;keyword&quot;, etc.)</li>
<li><code>value</code> - the collection value to filter on, defined in the YAML font-matter of pages (&quot;new&quot;, &quot;all&quot;, &quot;javascript&quot;, etc.)</li>
</ul>
<p><strong>Sorting</strong></p>
<p>There are two arguments for sorting attribute and direction:</p>
<ul>
<li><code>sortBy</code> - (optional) the page property to sort by, typically defined in the YAML front-matter of pages and prefixed
with &quot;data.&quot;, such as &quot;data.posted&quot;.  Default sorting is the collection default.</li>
<li><code>dir</code> - (optional) Sort direction, one of &quot;ascending&quot;, &quot;asc&quot;, &quot;descending&quot;, &quot;desc&quot;.  Ascending by default.</li>
</ul>
<p><strong>Limiting</strong></p>
<p>You can limit the number of pages selected as follows:</p>
<ul>
<li><code>limit</code>- (optional) the maximum number of pages to return</li>
</ul>
<p><strong>Diagnostics</strong></p>
<p>There are two arguments to help troubleshoot selecting pages:</p>
<ul>
<li><code>verbose</code> - (optional) true or false if you want verbose console logging.  Defaults to false.</li>
<li><code>name</code> - (optional) provide a unique name to this {{#withPages}} tag to help match verbose messages to markup.
Defaults to a combination of the arguments.</li>
</ul>
<h2 id="installation">Installation</h2>
<p>For now, copy the code below into the file in your helpers folder.
You do have a <a href="step-7-custom-handlebars-helpers.html">helpers folder</a>, right?  I might make this a plugin in the future.</p>
<pre><code class="language-javascript">(<span class="function"><span class="keyword">function</span><span class="params">()</span> {</span>
    <span class="string">"use strict"</span>;

    <span class="keyword">var</span> _ = require(<span class="string">"lodash"</span>);

    <span class="function"><span class="keyword">function</span> <span class="title">getPropertyFromSpec</span><span class="params">(obj, propertySpec)</span> {</span>
        <span class="keyword">var</span> properties = propertySpec.split(<span class="string">'.'</span>);
        <span class="keyword">var</span> resultObj = obj;
        _.each(properties, <span class="function"><span class="keyword">function</span> <span class="params">(property)</span> {</span>
            <span class="keyword">if</span> (resultObj &amp;&amp; resultObj[property]) {
                resultObj = resultObj[property];
            } <span class="keyword">else</span> {
                resultObj = <span class="literal">null</span>;
            }
        });
        <span class="keyword">return</span> resultObj;
    }

    module.exports.register = <span class="function"><span class="keyword">function</span><span class="params">(Handlebars, options)</span> {</span>
        Handlebars.registerHelper(<span class="string">"withPages"</span>, <span class="function"><span class="keyword">function</span><span class="params">(options)</span> {</span>
            <span class="keyword">var</span> result = <span class="string">""</span>;
            <span class="keyword">var</span> verbose = options.hash.verbose;
            <span class="keyword">var</span> collectionName = options.hash.collection;
            <span class="keyword">var</span> inflection = options.hash.inflection;
            <span class="keyword">var</span> inflectionValue = options.hash.value;
            <span class="keyword">var</span> limit = options.hash.limit;
            <span class="keyword">var</span> sortBy = options.hash.sortBy;
            <span class="keyword">var</span> sortDir = options.hash.dir;
            <span class="keyword">var</span> sortDescending = options.hash.dir &amp;&amp; (sortDir === <span class="string">"desc"</span> || sortDir === <span class="string">"descending"</span>);
            <span class="keyword">var</span> name = options.hash.name || <span class="string">""</span> + [collectionName, inflection, inflectionValue, sortBy, sortDir, limit].join(<span class="string">"/"</span>);
            <span class="keyword">var</span> sourcePageName = <span class="keyword">this</span>.page.src.substr(<span class="keyword">this</span>.page.src.lastIndexOf(<span class="string">"/"</span>) + <span class="number">1</span>);
            <span class="keyword">var</span> logPrefix = <span class="string">"{{#withPages}} "</span> + name + <span class="string">" in "</span> + sourcePageName + <span class="string">": "</span>;

            <span class="function"><span class="keyword">function</span> <span class="title">log</span><span class="params">(level, message, extra)</span> {</span>
                <span class="keyword">if</span> (verbose || level !== <span class="string">"info"</span>) {
                    <span class="keyword">if</span> (extra) {
                        console[level](logPrefix + message, extra);
                    } <span class="keyword">else</span> {
                        console[level](logPrefix + message);
                    }
                }
            }

            log(<span class="string">"info"</span>, <span class="string">"{{withPages}} options:"</span>, options.hash);
            <span class="keyword">if</span> (sortDir &amp;&amp; (sortDir !== <span class="string">"asc"</span> &amp;&amp; sortDir !== <span class="string">"ascending"</span> &amp;&amp; sortDir !== <span class="string">"desc"</span> &amp;&amp; sortDir !== <span class="string">"descending"</span>)) {
                log(<span class="string">"warn"</span>, <span class="string">"{{withPages dir parameter '"</span> + sortDir + <span class="string">"' is not recognized.  "</span> +
                    <span class="string">"Use one of 'asc', 'ascending', 'desc', 'descending'"</span>);
            }

            <span class="keyword">var</span> selectedPages = <span class="keyword">this</span>.pages;

            <span class="comment">// Collection filtering</span>
            <span class="keyword">if</span> (collectionName &amp;&amp; inflection &amp;&amp; inflectionValue) {
                <span class="keyword">var</span> collection = <span class="keyword">this</span>[collectionName];
                <span class="keyword">if</span> (!collection) {
                    log(<span class="string">"error"</span>, <span class="string">"collection '"</span> + collectionName + <span class="string">"' not found"</span>);
                    <span class="keyword">return</span> result;
                }
                <span class="keyword">if</span> (collection.length &gt; <span class="number">0</span> &amp;&amp; !collection[<span class="number">0</span>][inflection]) {
                    log(<span class="string">"warn"</span>, <span class="string">"{{withPages}} did not find pages matching inflection '"</span> + inflection +
                        <span class="string">"' in collection '"</span> + collectionName + <span class="string">"'"</span>);
                    <span class="keyword">return</span> result;
                }
                <span class="keyword">var</span> collectionFilter = {};
                collectionFilter[inflection] = inflectionValue;
                <span class="keyword">var</span> collectionSubset = _.find(collection, collectionFilter);
                <span class="keyword">if</span> (collectionSubset &amp;&amp; collectionSubset.pages &amp;&amp; collectionSubset.pages.length &gt; <span class="number">0</span>) {
                    log(<span class="string">"info"</span>, <span class="string">"{{withPages}} found "</span> + collectionSubset.pages.length + <span class="string">" pages in collection '"</span> +
                        collectionName + <span class="string">"' with inflection '"</span> + inflection + <span class="string">"' and value '"</span> + inflectionValue + <span class="string">"'"</span>);
                    selectedPages = collectionSubset.pages;
                } <span class="keyword">else</span> {
                    log(<span class="string">"info"</span>, <span class="string">"{{withPages}} did not find any pages in collection '"</span> + collectionName +
                        <span class="string">"' with inflection '"</span> + inflection + <span class="string">"' and value '"</span> + inflectionValue + <span class="string">"'"</span>);
                    <span class="keyword">return</span> result;
                }
            } <span class="keyword">else</span> {
                log(<span class="string">"info"</span>, <span class="string">"using all "</span> + selectedPages.length + <span class="string">" pages"</span>);
            }

            <span class="comment">// Sorting</span>
            <span class="keyword">if</span> (sortBy) {
                <span class="keyword">var</span> pagesWithSortByValue = <span class="number">0</span>;
                selectedPages = _.sortBy(selectedPages, <span class="function"><span class="keyword">function</span> <span class="params">(item)</span> {</span>
                    <span class="keyword">var</span> sortByValue = getPropertyFromSpec(item, sortBy);
                    <span class="keyword">if</span> (sortByValue) {
                        pagesWithSortByValue++;
                    }
                    <span class="keyword">return</span> sortByValue;
                });
                log(<span class="string">"info"</span>, <span class="string">"{{withPages}} sorted by '"</span> + sortBy + <span class="string">"'. "</span> + pagesWithSortByValue + <span class="string">" of "</span> +
                    selectedPages.length + <span class="string">" pages had '"</span> + sortBy + <span class="string">"' values."</span>);
                <span class="keyword">if</span> (pagesWithSortByValue &lt; selectedPages.length &amp;&amp; sortBy.indexOf(<span class="string">"data."</span>) == -<span class="number">1</span>) {
                    log(<span class="string">"info"</span>, <span class="string">"{{withPages}} Did you mean to use sortBy 'data."</span> + sortBy + <span class="string">"'?"</span>);
                }
            }
            <span class="keyword">if</span> (sortDescending) {
                selectedPages = selectedPages.reverse();
                log(<span class="string">"info"</span>, <span class="string">"{{withPages}} reversing sort order"</span>);
            }

            <span class="comment">// Limit</span>
            <span class="keyword">if</span> (limit &amp;&amp; limit &gt;= <span class="number">0</span>) {
                <span class="keyword">var</span> limitedPages = _.first(selectedPages, limit);
                log(<span class="string">"info"</span>, <span class="string">"{{withPages}} limiting to "</span> + limitedPages.length + <span class="string">" pages out of "</span> +
                    selectedPages.length);
                selectedPages = limitedPages;
            }

            <span class="comment">// Rendering</span>
            _.each(selectedPages, <span class="function"><span class="keyword">function</span> <span class="params">(page, index, pages)</span> {</span>
                log(<span class="string">"info"</span>, <span class="string">"{{withPages}} rendering page: "</span> + page.basename);
                result += options.fn(page);
            });

            <span class="keyword">return</span> result;
        });
    };

}).call(<span class="keyword">this</span>);</code></pre>
<p>For the code, please see the
<a href="https://github.com/jvwing/learning-assemble">learning-assemble</a> repository on GitHub.
The branch <a href="https://github.com/jvwing/learning-assemble/tree/step12-navigation-helper">step12-navigation-helper</a>
contains the completed code from this post.</p>

                

                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                
                    
                

            </div>

            <div class="share">
                <a href="https://twitter.com/share" class="twitter-share-button" data-text="Learning Assemble, Step 12 - Navigation Helper" data-via="jvwing" data-size="large">Tweet</a>
                <script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0],p=/^http:/.test(d.location)?'http':'https';if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src=p+'://platform.twitter.com/widgets.js';fjs.parentNode.insertBefore(js,fjs);}}(document, 'script', 'twitter-wjs');</script>
            </div>

            <hr/>
            <div class="other-posts">
                <h4>You might also enjoy:</h4>
                <ul>
                    
                        <li><a href="/learning-assemble/step-1-hello-assemble.html">Learning Assemble, Step 1 - Hello, Assemble!</a></li>
                    
                        <li><a href="/learning-assemble/step-2-basic-templating.html">Learning Assemble, Step 2 - Basic Templating</a></li>
                    
                        <li><a href="/learning-assemble/step-3-grunt-workflow.html">Learning Assemble, Step 3 - Grunt Workflow</a></li>
                    
                        <li><a href="/learning-assemble/step-4-data.html">Learning Assemble, Step 4 - Data</a></li>
                    
                        <li><a href="/learning-assemble/step-5-page-navigation.html">Learning Assemble, Step 5 - Page Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-6-blogging-with-markdown.html">Learning Assemble, Step 6 - Blogging with Markdown</a></li>
                    
                        <li><a href="/learning-assemble/step-7-custom-handlebars-helpers.html">Learning Assemble, Step 7 - Custom Handlebars Helpers</a></li>
                    
                        <li><a href="/learning-assemble/step-8-debugging-assemble.html">Learning Assemble, Step 8 - Debugging Assemble</a></li>
                    
                        <li><a href="/learning-assemble/step-9-sitemap.html">Learning Assemble, Step 9 - Sitemap</a></li>
                    
                        <li><a href="/learning-assemble/step-10-pages-from-data.html">Learning Assemble, Step 10 - Pages from Data</a></li>
                    
                        <li><a href="/learning-assemble/step-11-more-navigation.html">Learning Assemble, Step 11 - More Navigation</a></li>
                    
                        <li><a href="/learning-assemble/step-12-navigation-helper.html">Learning Assemble, Step 12 - Navigation Helper</a></li>
                    
                </ul>
            </div>

            <div class="comments">
                <div id="disqus_thread"></div>
<script type="text/javascript">
    /* * * CONFIGURATION VARIABLES * * */
    var disqus_shortname = 'jameswingnet';

    /* * * DON'T EDIT BELOW THIS LINE * * */
    (function() {
        var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
        dsq.src = '//' + disqus_shortname + '.disqus.com/embed.js';
        (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
    })();
</script>
<noscript>Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript" rel="nofollow">comments powered by Disqus.</a></noscript>
            </div>

        </div>

    </div>

</div>

        </div>
        <div class="col-md-3 col-lg-4 sidebar">
            <div class="posts-list-brief">
    <h3 class="sidebar-heading">Recent</h3>
    <ul>
        
            
        
            
                
                    <li>
                        
                            <a href="/aws/query-cloudtrail-events-with-athena.html">Query CloudTrail Events with Athena</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/nifi-internal-fields.html">Hidden Fields in Apache NiFi FlowFiles</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/nifi/first-look-at-apache-nifi.html">First Look at Apache NiFi</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/monitoring/monitoring-advice-distilled.html">Monitoring Advice Distilled</a>
                        
                    </li>
                
                    <li>
                        
                            <a href="/aws/using-codecommit-and-git-credentials.html">Using CodeCommit and GitHub Credential Helpers</a>
                        
                    </li>
                
            
        
            
        
            
        
            
        
            
        
    </ul>
</div>
            <hr/>
            <div class="ads">
    <!--<h3 class="sidebar-heading">Featured</h3>-->
    <div class="ad">
        <a href="http://prontolyzer.jameswing.net/">
            <img class="thumbnail" src="/assets/ads/prontolyzer-thumb.png" />
            <h4 class="title">Prontolyzer</h4>
            <p class="description">
                Interactive mapping and exploration of bike trip data for the 2015 Pronto Cycle Share Data Challenge.
                
            </p>
        </a>
    </div>
    <hr/>
    <div class="ad">
        <a href="/troubleshooting-chart/introducing-the-troubleshooting-chart.html">
            <img class="thumbnail" src="/assets/posts/troubleshooting-chart/troubleshootingchartannotated-thumbnail.png" />
            <h4 class="title">The Troubleshooting Chart</h4>
            <p class="description">
                Level up your troubleshooting with simple data analysis and visualizations.
                A thrill-ride for Ops types.
            </p>
        </a>
    </div>
</div>
        </div>
    </div>
</div>




<div id="footer">
    <div class="container">
        <div class="row">
            <div class="col-sm-6 copyright">
                © 2014-2017 James Wing. All Rights Reserved.
            </div>
            <div class="col-sm-6 menu">
                <ul>
                    

<li><a href="/index.html">Blog</a></li>
<li><a href="/about.html">About</a></li>


                </ul>
            </div>
        </div>
    </div>
</div>






<script src="//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.5/js/bootstrap.min.js"></script>

<script>
    (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
        (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
            m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
    })(window,document,'script','//www.google-analytics.com/analytics.js','ga');
    var gaId = 'UA-6573';
    gaId += '6738-1';
    ga('create', gaId, 'auto');
    ga('send', 'pageview');

</script>









<script src="../assets/prism/prism.js"></script>


</body>
</html>